"use strict";
exports.__esModule = true;
var Property_generated_1 = require("./Property.generated");
exports.property = Property_generated_1.property;
var Property_generic_1 = require("./Property.generic");
exports.Property = Property_generic_1.Property;
//# sourceMappingURL=Property.js.map