"use strict";
function __export(m) {
    for (var p in m) if (!exports.hasOwnProperty(p)) exports[p] = m[p];
}
exports.__esModule = true;
var fc = require("./fast-check-default");
exports["default"] = fc;
__export(require("./fast-check-default"));
//# sourceMappingURL=fast-check.js.map