"use strict";
exports.__esModule = true;
var AsyncProperty_generated_1 = require("./AsyncProperty.generated");
exports.asyncProperty = AsyncProperty_generated_1.asyncProperty;
var AsyncProperty_generic_1 = require("./AsyncProperty.generic");
exports.AsyncProperty = AsyncProperty_generic_1.AsyncProperty;
//# sourceMappingURL=AsyncProperty.js.map